require "defines"

GRID_SINGLE_LINE_SEGMENTS = 12
GRID_SIZE = 8
GRID_ENTITY_HORIZONTAL = "grid-horizontal"
GRID_ENTITY_VERTICAL = "grid-vertical"
GRID_INDEX_MAX = 3

script.on_event(defines.events.on_built_entity, function(event) 

    local player = game.get_player(event.player_index)
    local created_entity = event.created_entity

    if created_entity.name == "grid-emitter" then
        table.insert(global.ruler_emitters, { entity = created_entity, recipe_name = "", grid = {}})
    end

end)

glob_init = function()

    global.ruler_emitters = global.ruler_emitters or {}
    global.ruler_info = global.ruler_info or { grid_index = 1 }

end

clean_old_grid = function(entry)
 
    local i = 0

    for i = 1, #entry.grid do
        entry.grid[i].destroy()
    end

    entry.grid = {}

end

create_new_grid = function(entry)

    local entity = entry.entity
    local lines, distance = string.match(entry.recipe_name, "grid%-(%d+)%-(%d+)")

    if lines == nil or distance == nil then
        return
    end

    local ex = entity.position.x
    local ey = entity.position.y

    local x = 0
    local y = 0

    local line_length = 0

    lines = tonumber(lines)
    distance = tonumber(distance) + 1

    local lines2 = math.floor((lines - 1) / 2)

    if lines2 == 0 then
        line_length = GRID_SINGLE_LINE_SEGMENTS * 2 + 1
    else
        line_length = math.floor((2 * lines2 * distance) / GRID_SIZE) + 1
    end

    local line_shift = math.floor(((line_length - 1) * GRID_SIZE) / 2)

    local hx = ex - line_shift
    local hy = ey - lines2 * distance

    local vx = ex - lines2 * distance
    local vy = ey - line_shift

    for i = 1, line_length do
        for j = 1, lines do
        
            x = hx + (i - 1) * GRID_SIZE
            y = hy + (j - 1) * distance

            table.insert(entry.grid, game.get_surface(1).create_entity{
                name = GRID_ENTITY_HORIZONTAL .. "-" .. global.ruler_info.grid_index,
                position = { x = x, y = y }
            })

            x = vx + (j - 1) * distance
            y = vy + (i - 1) * GRID_SIZE

            table.insert(entry.grid, game.get_surface(1).create_entity{
                name = GRID_ENTITY_VERTICAL .. "-" .. global.ruler_info.grid_index,
                position = { x = x, y = y }
            })

        end
    end

    global.ruler_info.grid_index = global.ruler_info.grid_index + 1
    if global.ruler_info.grid_index > GRID_INDEX_MAX then
        global.ruler_info.grid_index = 1
    end

end

switch_recipe = function(recipe_name, entry)

    local entity = entry.entity
    
    clean_old_grid(entry)
    entry.recipe_name = recipe_name
    create_new_grid(entry)

end

process_ruler_emitters = function()

    local i = 0

    for i = 1, #global.ruler_emitters do

        local recipe_name = ""
        local entry = global.ruler_emitters[i]

        if entry.entity.energy == 0 and entry.recipe_name ~= "" then
            recipe_name = ""
        end

        if entry.entity.energy > 0 and entry.entity.recipe ~= nil then
            recipe_name = entry.entity.recipe.name
        end

        if recipe_name ~= entry.recipe_name then
            switch_recipe(recipe_name, entry)
        end

    end

end

script.on_event(defines.events.on_tick, function(event)

    if event.tick % 150 then
        if #global.ruler_emitters > 0 then
            process_ruler_emitters()
        end
    end

end)

script.on_event(defines.events.on_preplayer_mined_item, function(event)

    local entity = event.entity

    if entity.name == 'grid-emitter' then

        local index = 1
        local found = false

        while index <= #global.ruler_emitters do

            local entry = global.ruler_emitters[index] 
            local pos = entry.entity.position

            if pos.x == entity.position.x and pos.y == entity.position.y then
                found = true
                break
            end

            index = index + 1

        end

        if not found then
            return
        end

        clean_old_grid(global.ruler_emitters[index])
        table.remove(global.ruler_emitters, index)

    end

end)

script.on_init(function()

    glob_init()

end)

script.on_load(function()

    glob_init()

end)
